#
#    MRTG Webmin Module Library
#    Copyright (C) 2000 by Joern Lammers, KIM Systemhaus GmbH, Braunschweig
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    Created  : 2000-11-02
#    Changed  : 2001-10-31
#               mrtg executable, cfgmaker an indexmaker may all be either in searchpath or mrtg_scr_path
#               indexmaker syntax has changed again (latest version now 2.9.17)
#               added use_cron option
#    Changed  : 2003-03-16
#               MRTG version autodetect added
#               support module clones
#                 remove "mrtg" from init_config
#                 use $module_name for links
#               added acl support
#    Changed  : 2007-06-23
#               use icondir with indexmaker
#		flush_files_lines was called with wrong parameter (resulting in a broken cfg)



# $debug=1;


do '../web-lib.pl';

&init_config();
%access = &get_module_acl();

# check, check, testing, 1, 2.......

# critical errors
# look if everything we need is there (or somewhere else)
$err=0;
$err_msg="";
# the executables may be in a standard path or the provided scripts path
foreach (split(/:/, "$ENV{PATH}:$config{'mrtg_scr_path'}")) {
 if (-x "$_/mrtg") {$mrtg_cmd=$_};
 if (-x "$_/cfgmaker") {$cfgm_cmd=$_};
 if (-x "$_/indexmaker") {$idxm_cmd=$_};
}
if (!$mrtg_cmd) {
    $err_msg.=&text('err_miss_ex')."<BR>";
    $err++ }
if (!$cfgm_cmd) {
    $err_msg.=&text('err_miss_cfgmaker')."<BR>";
    $err++ }
if (!$idxm_cmd) {
    $err_msg.=&text('err_miss_indexmaker')."<BR>";
    $err++ }
# mrtg images path
if (!-d "$config{'mrtg_img_path'}") { 
    $err_msg.=&text('err_miss_img_p')."<BR>";
    $err++ }
# mrtg configuration path (configuration file will be webmin.cfg)
if (!-d "$config{'mrtg_cfg_path'}") { 
    $err_msg.=&text('err_miss_cfg_p')."<BR>";
    $err++ }
# mrtg output path
if (!-d "$config{'mrtg_htm_path'}") { 
    $err_msg.=&text('err_miss_htm_p')."<BR>";
    $err++ }
# we need the cron module
if ((&foreign_check("cron")==0) && ($config{use_cron}==1)){
    $err_msg.=&text('err_miss_cron')."<BR>"; 
    $err++ }
# stop on critical errors
if ($err>0) { 
    $err_msg=$err." ".&text('errors')."<BR>".$err_msg." <A HREF=\"/config.cgi?$module_name\">".&text('config_link')."</A>";
    &error($err_msg) }

#
# non critical
#

#
# try to guess mrtg version if not supplied and set config
#
if ($config{'mrtg_ver'} eq "") {
 system("$mrtg_cmd/mrtg 2>&1 >$config{'mrtg_cfg_path'}/err.cfg");
 $ver=(grep {/^mrtg-/} &mrtg_err_get)[0];
 $ver=(split / /, substr($ver,5,10))[0];
# hope it's safe to use read_file and write_file for config.....
 &read_file("$module_config_directory/config", \%lines);
 if (exists $lines{'mrtg_ver'}) {
  $lines{'mrtg_ver'}=$ver;
  &write_file("$module_config_directory/config", \%lines);
  $config{'mrtg_ver'}=$ver;
  $err_msg.=&text('err_vers')." $ver<BR>"; 
  }
 }
@l=split /\./,$config{'mrtg_ver'};
$ver=1000000*$l[0]+1000*$l[1]+$l[2];

#
# automatic reconfiguration of symlink to html data
# also set WorkDir in MRTG configuration
$s_link="./data-$module_name";
$o_link=readlink($s_link);
$n_link=$config{'mrtg_htm_path'};
if ( $o_link ne $n_link) {
 $err_msg.=&text('err_htm_change')." ".$n_link."<BR>";
 unlink($s_link);
 symlink($n_link,$s_link); 
 &mrtg_cfg_setwdir;
 }

#
# automatic cron management
# if cron is to be used
if ($config{use_cron}==1){
 $mrtg_cmd.="/mrtg $config{'mrtg_cfg_path'}/webmin.cfg";
 &foreign_require("cron", "cron-lib.pl");
 @jobs = &foreign_call( "cron", "list_cron_jobs");
 $i=0;
 foreach (map{$$_{'command'}} @jobs){
    if ($_ eq $mrtg_cmd) {$i++};
 }
# too many correct cron entries
 if ($i>1) {
  $err_msg.=&text('err_cron_mult')."<BR>";
  } 
# add cron entry for current configuration
# we don't delete old entries. the user will have to do that  
 if ($i==0) {
  $mrtg_job={'mins'    =>"1,6,11,16,21,26,31,36,41,46,51,56",
             'hours'   =>"*",
             'days'    =>"*",
             'months'  =>"*",
             'weekdays'=>"*",
             'user'    =>"root",
             'active'  =>1,
             'type'    =>0,
             'command' =>$mrtg_cmd};
  &foreign_call("cron", "create_cron_job", $mrtg_job);
  $err_msg.=&text('err_cron_add')."<BR>";
  }
}
#
# copy mrtg images if needed
if ($ver < 2009000) {
 if (! glob("$config{'mrtg_htm_path'}/mrtg-*.gif")) {
  foreach $in (glob("$config{'mrtg_img_path'}/mrtg-*.gif")) {
   @list=split /\//,$in;
   $out="$config{'mrtg_htm_path'}/$list[@list-1]";
   open IN, "<$in";
   open OUT,">$out";
   &copydata(IN,OUT);
   close OUT;
   close IN;
   }
  $err_msg.=&text('err_gif_copy')."<BR>";
  }
}
if ($ver > 2008008) { 
 if (! glob("$config{'mrtg_htm_path'}/mrtg-*.png")) {
  foreach $in (glob("$config{'mrtg_img_path'}/mrtg-*.png")) {
   @list=split /\//,$in;
   $out="$config{'mrtg_htm_path'}/$list[@list-1]";
   open IN, "<$in";
   open OUT,">$out";
   &copydata(IN,OUT);
   close OUT;
   close IN;
   }
  $err_msg.=&text('err_png_copy')."<BR>";
  }
 }



  

##########
# MRTG subs
#

# remove all WorkDir entries and set WorkDir 
sub mrtg_cfg_setwdir {
 $cfg=&read_file_lines("$config{'mrtg_cfg_path'}/webmin.cfg");
 @$cfg=grep {!/^WorkDir:/} @$cfg;
 unshift @$cfg,"WorkDir:$config{'mrtg_htm_path'}";
 &flush_file_lines("$config{'mrtg_cfg_path'}/webmin.cfg");
 }

# delete current configuration
sub mrtg_cfg_del {
 $cfg=&read_file_lines("$config{'mrtg_cfg_path'}/webmin.cfg");
 @$cfg=("WorkDir:$config{'mrtg_htm_path'}");
 &flush_file_lines("$config{'mrtg_cfg_path'}/webmin.cfg");
 }


# add snmp host with cfgmaker
#   mrtg_cfg_addhost(community,hostname)
sub mrtg_cfg_addhost {
 system("$cfgm_cmd/cfgmaker $_[0]\@$_[1] >>$config{'mrtg_cfg_path'}/webmin.cfg 2>$config{'mrtg_cfg_path'}/err.cfg"); 
 &mrtg_cfg_setwdir;
 }


# run indexmaker
# indexmaker does not return errors in STDERR check!!
sub mrtg_cfg_makeindex {
 $title=&text('view_title');
 $title_1=&text('view_title_1');
 if ($ver >= 2010013) {
  system("$idxm_cmd/indexmaker --columns=1 --icondir=. --title=\"$title\" $config{'mrtg_cfg_path'}/webmin.cfg >$config{'mrtg_htm_path'}/index.html 2>$config{'mrtg_cfg_path'}/err.cfg"); 
  } elsif ($ver >= 2009000) {
  system("$idxm_cmd/indexmaker --columns=1 --title=\"$title\" $config{'mrtg_cfg_path'}/webmin.cfg >$config{'mrtg_htm_path'}/index.html 2>$config{'mrtg_cfg_path'}/err.cfg"); 
  } elsif ($ver >= 2007000) {
  system("$idxm_cmd/indexmaker -1 -t \"$title_1\" $config{'mrtg_cfg_path'}/webmin.cfg >$config{'mrtg_htm_path'}/index.html 2>$config{'mrtg_cfg_path'}/err.cfg"); 
  } else { 
  system("$idxm_cmd/indexmaker $config{'mrtg_cfg_path'}/webmin.cfg \"$title\" ^ >$config{'mrtg_htm_path'}/index.html 2>$config{'mrtg_cfg_path'}/err.cfg"); 
  }
}

# return list containing error output
sub mrtg_err_get {
 @list=(); 
 $in="$config{'mrtg_cfg_path'}/err.cfg";
 if (-e $in) {
  open IN,"<$in";
  while($line=<IN>) {push @list,$line};
  close IN;
  }
 return @list;
 } 

###########################################################################
#  DEBUGGING STUFF....  ignore if I forget to delete this
#
#
#
if ($debug){
$err_msg.="<table border=1>";
$err_msg.="<tr>";
$err_msg.="<td>ACL: ".$access{'allow_config'}."</td>";
$err_msg.= "</tr><tr>";
$err_msg.= "<td>Module Name: ".$module_name."</td>";
$err_msg.= "</tr>";
$err_msg.= "</table>";
}
#
#
###########################################################################


1;
